#ifndef _MESH_H
#define _MESH_H

#include <vector>
#include <set>
#include "Vector3.h"
using namespace std;
class Node
{
public:
	Node(double coor_x, double coor_y, double coor_z)
	{
		x = coor_x;	y = coor_y; z = coor_z;
		boundary_node = false;
	}

public:
	double  x, y, z;
	bool boundary_node;
};

class Hedge
{
public:
	Hedge(int from, int to, int f)
	{
		fromNode = from;
		toNode = to;
		face = f;
	}
public:
	int fromNode;
	int toNode;
	int face;
};

class Mesh
{
public:
	Mesh()
	{
		mesh_node.clear();
		xmax = -1.0*exp(20.0);
		xmin = 1.0*exp(20.0);
		ymax = -1.0*exp(20.0);
		ymin = 1.0*exp(20.0);
		zmax = -1.0*exp(20.0);
		zmin = 1.0*exp(20.0);
	}
	//init the one ring neighbors faces of each face, each node. Also mark the boundary nodes.
	void initMeshInfo();

	void init_alive_face()
	{
		face_alive = vector<bool>(mesh_face.size(), true);
		cluster_pos = vector<vec3>(mesh_face.size(), vec3(0,0,0));
		for (int i=0; i< mesh_face.size(); i++)
		{
			int v1 = mesh_face[i].x;
			int v2 = mesh_face[i].y;
			int v3 = mesh_face[i].z;
			cluster_pos[i] = vec3((mesh_node[v1].x+ mesh_node[v2].x + mesh_node[v3].x)/3.0, (mesh_node[v1].y+ mesh_node[v2].y + mesh_node[v3].y)/3.0, (mesh_node[v1].z+ mesh_node[v2].z + mesh_node[v3].z)/3.0);
		}
	}

	bool face_is_valid(int i)	{ return face_alive[i]; }
	void mark_invalid(int i)	{ face_alive[i] = false; }
	void set_position(int i, double x_coor, double y_coor, double z_coor)	{ cluster_pos[i].x = x_coor; cluster_pos[i].y = y_coor; cluster_pos[i].z = z_coor; }
	vector<int> getFaceNeighbors(int i)
	{
		return face_neighbors[i];
	}
public:
	vector<Node> mesh_node;
	vector<Vector3<long int>> mesh_face;
	vector<vec3> cluster_pos;

	vector<bool> face_alive; 
	vector<vector<int>> face_neighbors; // one ring neighbor faces of each face
	vector<vector<int>> node_neighbors; // one ring neighbor faces of each node
	//bounding box
	double xmax;
	double xmin;
	double ymax;
	double ymin;
	double zmax;
	double zmin;
};
#endif 