#ifndef PCCVT_INCLUDED // -*- C++ -*-
#define PCCVT_INCLUDED
#if !defined(__GNUC__)
#  pragma once
#endif

#include "CovDeterminant.h"
#include "mesh.h"
#include <set>
#include <vector>
#include "MxStdSlim.h"

typedef int FaceID;
typedef int ClusterID;
const float energy_convergence = 0.00001;
using namespace std;

class face_pair : public MxHeapable
{
public:
	FaceID f1, f2;
};  

//face re-arrangement
class face_swap: public MxHeapable
{
public:
	ClusterID from_id, to_id;
	FaceID face_id;

	face_swap(){ }
};

class Partition
{
public:
    uint D;
	MxHeap heap;

	vector<CovDet> __cov;
	vector<CovDet> initial_cov;
	vector<vector<face_pair*>> face_neighbor_links; 
	vector<vector<face_pair>> initface_neighbor_links;

	typedef face_pair domain_pair;
	vector<vector<domain_pair>> cluster_neighbor_links;
	vector<double> cluster_energy;
	vector<set<FaceID>> cluster_boundary_faces;

	vector<face_swap *> info_set;
	bool swap_started;
	double energy_previous_iter;

public:
    Partition();
	~Partition();
	void init_cov();
	bool decimate(uint target);

	void init_face_swap();
	bool swap_face_once();
	void output_partition_ply();
	float print_total_energy();
	//aux
	vector<int> find_neighbor_clusters(int before_mapping);
	int find_cluster_id(int after_mapping);	
public:
	void collect_triangles_covariance();
	void collect_face_pair();
	void create_pair(FaceID j, FaceID k);
	void compute_pair_info(face_pair* info);
	void compute_energy(face_pair *info);
	void finalize_pair_update(face_pair *info);
	void erase_pair_counterpart(ClusterID u, face_pair *info);

	void apply_contraction(face_pair *info);
	void update_valid_pairs(face_pair *info);
	int find_contracted_neighbors(FaceID v1, set<FaceID> & neighbors);

	void prepare_space();	
	void update_cluster_information(ClusterID j);
	void collect_cluster_neighbor();
	void collect_cluster_swaps(ClusterID i);
	double calc_energy_delta(ClusterID from, ClusterID to, FaceID face_id, face_swap *info);
	void store_swap(float energy_saved, ClusterID from, ClusterID to, FaceID face_id, face_swap *info);

	void apply_swap_set(vector<face_swap *> info_set, set<ClusterID>& recollect_domain);
	void apply_swap(face_swap* info, set<ClusterID>& update_domain);
public:
	uint dim() const { return D; }
	uint cov_count() const { return __cov.size(); }
	CovDet&       covariance(uint i)       { return __cov[i]; }
	const CovDet& covariance(uint i) const { return __cov[i]; }
	CovDet&       init_covariance(uint i)       { return initial_cov[i]; }
	const CovDet& init_covariance(uint i) const { return initial_cov[i]; }
};

#endif
