#ifndef REMESH_H
#define REMESH_H

#include "mesh.h"
#include <iterator>
#include "dijkstra.h"
class Anchor_node
{
public:
	Anchor_node(set<int> clusters, set<int> pid)
	{
		meeting_clusters = clusters;
		init_nodes_candidates = pid;
		closest_node = -1;
		pos = vec3(0,0,0);
		boundary = false;
	}
	Anchor_node(set<int> clusters, int pid)
	{
		meeting_clusters = clusters;
		init_nodes_candidates.clear();
		init_nodes_candidates.insert(pid);
		closest_node = -1;
		pos = vec3(0,0,0);
		boundary = false;
	}
public:
	set<int> meeting_clusters;
	set<int> init_nodes_candidates;
	mutable int closest_node;

	vec3 pos;
	vec3 n;
	bool boundary;
	int node_id; // trace back info in mesh construction
};

class AnchorCompare
{
public:
	bool operator()(const Anchor_node a1, const Anchor_node a2)
	{
		return a1.meeting_clusters < a2.meeting_clusters;
	}
};

class Edge
{
public:
	Edge(int node, int neighbor)
	{
		v1 = __min(node, neighbor);
		v2 = __max(node, neighbor);
		used_in_loop = false;
	}
public:
	int v1;
	int v2;
	bool used_in_loop;
};

class Proxy
{
public:
	Proxy()
	{
		neighboring_cluster_info.clear();
		edges.clear();
		loop.clear();
		prox_avg_n = vec3(0, 0, 0);
	}
public:
	vector<Anchor_node> neighboring_cluster_info; // used to extract edge info
	vector<Edge> edges;
	vector<int> loop;
	 vec3 prox_avg_n;
};

class Remesh
{
public:
	Remesh();
	//guarantee each cluster has one connected component, for a cluster has more than one connected component	
	//(1) hold the largest one as a cluster
	//(2) split each connected component as a new cluster, let its neighbor clusters be the merge candidata, choose the least energy pair to merge
	void cleanClusters(); 

	void construct_poly_mesh();
	void construct_CDT_mesh();
protected:
	void find_anchor_vertices();
	void project_anchor_vertices();
	void init_nodes();
	void edges_extraction();
	void loops_extraction();
	void orient_loops();
	void output_poly_mesh();

	void extract_triangles(int cluster_id, Dijkstra *dijkstra);
	void extract_all_triangles();
	void output_CDT_mesh();
	//aux
protected:
	int getMeetingClusters(int faceid, set<int> & clusterset);
	void mergeKSubset(set<Anchor_node, AnchorCompare>& anchor,int s[],int p,int k, int t[],int q,int r);

public:
	set<Anchor_node, AnchorCompare> anchor_nodes;
	vector<vec3> nodes;
	vector<vec3> nodes_n;
	vector<Vector3<long int>> triangle_faces;
	vector<Proxy> prox;
	int poly_num;

	Dijkstra *dijkstra;
	bool poly_mesh_ready;

	set<int> aaa_triangles;
};
#endif